module PUBMED.Client where

import Data.ByteString.Lazy (ByteString)
import Data.Proxy (Proxy(..))
import Servant.API
import Servant.Client
import qualified Data.Text as T
import qualified Network.HTTP.Media as M

import PUBMED.Types (APIKey, Query, PerPage)

data DB = PUBMED

newtype BsXml = BsXml ByteString
  deriving (Show)

instance Accept BsXml where
  contentType _ = "text" M.// "xml"

instance MimeUnrender BsXml BsXml where
  mimeUnrender _ = Right . BsXml

type PUBMEDAPI =
  "esearch.fcgi"
--  :> QueryParam "db" DB
-- not mandatory since the base db is pubmed
  :> QueryParam "api_key" APIKey
  :> QueryParam "term" T.Text
  :> QueryParam "retstart" Integer
  :> QueryParam "retmax" Integer
  :> Get '[BsXml] BsXml
  :<|>
  "efetch.fcgi"
  :> QueryParam "api_key" APIKey
  :> QueryParam "db" T.Text
  :> QueryParam "rettype" T.Text
  :> QueryParams "id" Integer
  :> Get '[BsXml] BsXml

pubmedApi :: Proxy PUBMEDAPI
pubmedApi = Proxy

search :: Maybe APIKey
       -> Maybe T.Text
       -> Maybe Integer
       -> Maybe Integer
       -> ClientM BsXml

fetch :: Maybe APIKey
      -> Maybe T.Text
      -> Maybe T.Text
      -> [Integer]
      -> ClientM BsXml
search :<|> fetch = client pubmedApi
